#!/usr/bin/perl

use strict;

use CGI; # Libreria CGI de Perl

# Recoge los parametros del formulario
my $form = new CGI;
my $seq = $form->param('secuencia');

# Encabezamiento http
print $form->header;

print <<HTML_TEXT;
<html>
<head>
  <meta content="text/html; charset=ISO-8859-1" http-equiv="content-type">
  <title>Traduccin de la secuencia</title>
</head>
<body>
HTML_TEXT

$seq =~ s/[\s\n\r]//g; # eliminar espacios y saltos de lnea
$seq = uc $seq; # pasar a mayscylas

# Comprobar si los parmetros son nmeros
if ($seq =~ /[^ACGT \n\r]/) {
  &error ("La secuencia de ADN es incorrecta");
  exit;
} else {
  for (my $cadena = 0; $cadena <= 1; $cadena++) {
    if ($cadena == 1) {
      $seq =~ tr/ACGT/TGCA/;
      $seq = reverse $seq;
    }
    for (my $pos = 0; $pos < 3; $pos++) {
      my $s = substr($seq, $pos);
      my $aa = &translate($s);
      print "<p>ORF";
      print "-" if $cadena == 1;
      print $pos + 1;
      print ":<br>$aa</p>\n";
    }
  }
}

print "</body></html>\n";

exit;

# Subrutinas
############
sub error () {
  my ($text) = @_;

  print "<p>Error! $text</p>\n";
  print "</body></html>\n";

  exit;
}

sub translate () {
  my ($s) = @_;

  # Cdigo gentico
  my (%cg) = (
    'TCA' => 'S', # Serina
    'TCC' => 'S', # Serina
    'TCG' => 'S', # Serina
    'TCT' => 'S', # Serina
    'TTC' => 'F', # Fenilalanina
    'TTT' => 'F', # Fenilalanina
    'TTA' => 'L', # Leucina
    'TTG' => 'L', # Leucina
    'TAC' => 'Y', # Tirosina
    'TAT' => 'Y', # Tirosina
    'TAA' => '*', # Stop
    'TAG' => '*', # Stop
    'TGC' => 'C', # Cisteina
    'TGT' => 'C', # Cisteina
    'TGA' => '*', # Stop
    'TGG' => 'W', # Triptofano
    'CTA' => 'L', # Leucina
    'CTC' => 'L', # Leucina
    'CTG' => 'L', # Leucina
    'CTT' => 'L', # Leucina
    'CCA' => 'P', # Prolina
    'CCC' => 'P', # Prolina
    'CCG' => 'P', # Prolina
    'CCT' => 'P', # Prolina
    'CAC' => 'H', # Histidina
    'CAT' => 'H', # Histidina
    'CAA' => 'Q', # Glutamina
    'CAG' => 'Q', # Glutamina
    'CGA' => 'R', # Arginina
    'CGC' => 'R', # Arginina
    'CGG' => 'R', # Arginina
    'CGT' => 'R', # Arginina
    'ATA' => 'I', # Isoleucina
    'ATC' => 'I', # Isoleucina
    'ATT' => 'I', # Isoleucina
    'ATG' => 'M', # Methionina
    'ACA' => 'T', # Treonina
    'ACC' => 'T', # Treonina
    'ACG' => 'T', # Treonina
    'ACT' => 'T', # Treonina
    'AAC' => 'N', # Asparagina
    'AAT' => 'N', # Asparagina
    'AAA' => 'K', # Lisina
    'AAG' => 'K', # Lisina
    'AGC' => 'S', # Serina
    'AGT' => 'S', # Serina
    'AGA' => 'R', # Arginina
    'AGG' => 'R', # Arginina
    'GTA' => 'V', # Valina
    'GTC' => 'V', # Valina
    'GTG' => 'V', # Valina
    'GTT' => 'V', # Valina
    'GCA' => 'A', # Alanina
    'GCC' => 'A', # Alanina
    'GCG' => 'A', # Alanina
    'GCT' => 'A', # Alanina
    'GAC' => 'D', # Acido Aspartico
    'GAT' => 'D', # Acido Aspartico
    'GAA' => 'E', # Acido Glutamico
    'GAG' => 'E', # Acido Glutamico
    'GGA' => 'G', # Glicina
    'GGC' => 'G', # Glicina
    'GGG' => 'G', # Glicina
    'GGT' => 'G', # Glicina
  );

  my $aa;
  for (my $x = 0; $x <= length($s)-1; $x += 3) {
    $aa .= $cg{substr($s, $x, 3)};
  }

  return $aa;
}
