#!/usr/bin/perl

use strict;

use CGI; # Libreria CGI de Perl

# Recoge los parametros del formulario
my $form = new CGI;
my $coo = $form->param('posicion');  # coordenada
my $org = $form->param('organismo'); # organismo

# GTF files
my %ORG = ("humano" => "Homo_sapiens.GRCh37.71.gtf",
           "perro" => "Canis_familiaris.CanFam3.1.71.gtf", 
           "gato" => "Felis_catus.Felis_catus_6.2.71.gtf");

my $GTF = "/home/user1/gtfs/" . $ORG{$org};

# Encabezamiento http
print $form->header;

print <<HTML_TEXT;
<html>
<head>
  <meta content="text/html; charset=ISO-8859-1" http-equiv="content-type">
  <title>Comprobacin de posiciones genmicas</title>
</head>
<body>
HTML_TEXT

# Comprobar si la coordenada es correcta, y si lo es recoge cromosoma y posicin
my $cro;
my $pos;
if ($coo =~ /^chr(.+):(\d+)$/) {
  $cro = $1;
  $pos = $2;
}  else {
  &error ("La posicion no es correcta. Por favor, usa el formato recomendado.");
  exit;
}



# Recorrer el fichero GTF
my $previo; # guarda el elemento genmico previo
my $found; # para controlar si la posicin est ms all del ltimo gen
open GTF, $GTF;
while (<GTF>) {
  chomp;

  my ($cromosoma, $elemento, $p1, $p2, $gen) = (split /\t/)[0, 2, 3, 4, 8];
  next unless $cromosoma eq $cro && $elemento eq "exon"; # no hacer nada hasta llegar a un exn en el cromosoma correcto
  $gen =~ m/gene_id \"(.+?)\".+exon_number \"(.+?)\".+gene_name \"(.+?)\"/;
  $gen = $1;
  my $n_exon = $2;
  my $name = $3;

  if ($pos >= $p1 && $pos <= $p2) { # si est dentro de un gen
    print "<p>La posicin cae dentro del exn $n_exon del gen $gen($name)</p>\n";
    $found = 1; last;
  } elsif ($pos < $p1) { # en la regin anterior
    if (!$previo) { # si est delante del primer gen del cromosoma
      print "<p>La posicin cae entre el inicio del cromosoma y el exn $n_exon del gen $gen($name)</p>\n";
      $found = 1; last;
    }

    my ($n_exon_pre, $gen_pre) = split /\t/, $previo; # si est entre 2 exones
    print "<p>La posicin cae entre el exn $n_exon_pre del gen $gen_pre y el exn $n_exon del gen $gen($name)</p>\n";
    $found = 1; last;
  }  
  $previo = "$n_exon\t$gen($name)";
}
close GTF;

# Si no se ha encontrado la coordenada, estar ms all del ltimo gen del cromosoma
if (!$found) {  
  print "<p>La posicin cae ms all del ltimo gen del cromosoma</p>\n";
}

# Etiquetas finales
print "</body></html>\n";

exit;

# Subrutinas
############
sub error () {
  my ($text) = @_;

  print "<p>Error! $text</p>\n";
  print "</body></html>\n";

  exit;
}

